/***************************************************************************
 *                                                                         *
 *   YANN - Yet Another Neural Network                                     *
 *   =================================                                     *
 *                                                                         *
 *   Copyright (C) Juan Pedro Bolivar Puente 2008                          *
 *                                                                         *
 *   This program is free software: you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *                                                                         *
 ***************************************************************************/

#include "network.h"
#include "cleaner.h"

using namespace std;

namespace yann
{

/*
 * TODO: ¡¡ Esta funcion no hace chequeo de errores en el fichero !!
 */
Network::Network (istream& in)
{
    int n_nodes;
    int n_conn;
    int n_layer;
    int node;
    int i, j;
    Value val;
    
    in >> Cleaner () >> n_nodes;
    m_nodes.resize (n_nodes);

    for (i = 0; i < n_nodes; ++i) {
	in >> Cleaner () >> val;
	m_nodes[i].setBias (val);
	in >> Cleaner () >> n_conn;
	for (j = 0; j < n_conn; ++j) {
	    in >> Cleaner () >> node
	       >> Cleaner () >> val;
#ifdef DEBUG
	    cout << node << " " << val << endl;
#endif
	    m_nodes[i].connect (Perceptron::Link (&m_nodes [node-1], val));
	}
    }

    in >> Cleaner () >> n_layer;
    m_layers.resize (n_layer+1);
    
    /* Capa de entrada */
    in >> Cleaner () >> n_nodes;
    m_input.resize (n_nodes);
    for (i = 0; i < n_nodes; ++i) {
	in >> Cleaner () >> node;
	m_nodes[node-1].connect (Perceptron::Link(&m_input [i], 1.0));
	m_input_task.addInput (&m_input [i]);
	m_layers[2].addTask (&m_nodes[node-1]);
    }
    m_layers[0].addTask (&m_input_task);
    m_layers[0].addNext (&m_layers [1]);
    m_layers[1].addNext (&m_layers [2]);
    
    /* Capas ocultas */
    for (i = 2; i < n_layer; ++i) {
	in >> Cleaner () >> n_nodes;
	for (j = 0; j < n_nodes; ++j) {
	    in >> Cleaner () >> node;
	    m_layers[i].addTask (&m_nodes [node-1]);
	}
	m_layers[i].addNext (&m_layers [i+1]);
    }
    
    /* Capa de salida */
    in >> Cleaner () >> n_nodes;
    for (j = 0; j < n_nodes; ++j) {
	in >> Cleaner () >> node;
	m_layers[i].addTask (&m_nodes [node-1]);
	m_output_task.addOutput (&m_nodes [node-1]);
	m_nodes [node-1].setOutputFunction (Perceptron::thresholdFunction);
    }
    m_layers[i].addTask (&m_output_task);
}

void Network::run (std::istream& in,
		   std::ostream& out)
{
    int i;
    
    m_input_task.setStream (&in);
    m_output_task.setStream (&out);

    for (i = 0; i < m_layers.size (); ++i)
	m_layers [i].start ();
    
    m_layers[m_layers.size () - 1].wait ();
}

} /* namespace yann */
