/***************************************************************************
 *                                                                         *
 *   YANN - Yet Another Neural Network                                     *
 *   =================================                                     *
 *                                                                         *
 *   Copyright (C) Juan Pedro Bolivar Puente 2008                          *
 *                                                                         *
 *   This program is free software: you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include "pipeline.h"

using namespace std;

namespace yann
{

void Pipeline::addTask (Task* task)
{
    m_task.push_back (task);
}
    
void Pipeline::addNext (Pipeline* next)
{
    m_next.push_back(next);
    next->m_prev.push_back (this);
}

void Pipeline::start ()
{
    m_started = true;
    m_thread.start (*this);
}

void Pipeline::run ()
{
    while (!m_must_finish)
    {
	waitPrevious ();
	waitNext ();
	if (!m_must_finish)
	    m_must_finish = update ();
	signalNext ();
	signalPrevious ();
    }

    m_must_finish = false;
    m_started = false;
}

bool Pipeline::update ()
{
    bool ret = false;
    
    for (list<Task*>::iterator it = m_task.begin ();
	 it != m_task.end();
	 ++it)
	ret = ret || (*it)->update ();

    return ret;
}

bool Pipeline::waitNext ()
{
    m_next_lock.lock ();
    while (m_next_not_finished > 0)
	m_next_cond.wait (m_next_lock);
    m_next_not_finished = m_next.size ();
    m_next_lock.unlock ();
    return false;
}

bool Pipeline::waitPrevious ()
{
    m_prev_lock.lock ();
    while (m_prev_finished < (signed) m_prev.size ())
	m_prev_cond.wait (m_prev_lock);
    m_prev_finished = 0;
    m_prev_lock.unlock ();
    
    return false;
}

void Pipeline::signalNext ()
{
    std::list<Pipeline*>::iterator it = m_next.begin ();
    while (it != m_next.end ()) {
	(*it)->m_must_finish = m_must_finish;
	(*it)->m_prev_finished++;
	(*it)->m_prev_cond.broadcast ();
	++it;
    }
}

void Pipeline::signalPrevious ()
{
    std::list<Pipeline*>::iterator it = m_prev.begin ();
    while (it != m_prev.end ()) {
	(*it)->m_next_not_finished--;
	(*it)->m_next_cond.broadcast ();
	++it;
    }
}

} /* namespace yann */
