/***************************************************************************
 *                                                                         *
 *   YANN - Yet Another Neural Network                                     *
 *   =================================                                     *
 *                                                                         *
 *   Copyright (C) Juan Pedro Bolivar Puente 2008                          *
 *                                                                         *
 *   This program is free software: you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation, either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *                                                                         *
 ***************************************************************************/

#ifndef YANN_PIPELINE_H
#define YANN_PIPELINE_H

#include <list>

#include "thread.h"
#include "mutex.h"
#include "task.h"

namespace yann
{

class Pipeline : public Runnable
{
    Thread m_thread;
    bool m_started;
    bool m_must_finish;
    
    std::list <Task*> m_task;
    
    std::list <Pipeline*> m_next;
    std::list <Pipeline*> m_prev;

    int m_prev_finished;
    int m_next_not_finished;
    Mutex m_prev_lock;
    Mutex m_next_lock;
    Condition m_next_cond;
    Condition m_prev_cond;
    
    void run ();    

    bool update ();
    bool waitNext ();
    bool waitPrevious ();
    void signalNext ();
    void signalPrevious ();
    
public:
    Pipeline () :
	m_thread (*this),
	m_started (false),
	m_must_finish (false),
	m_prev_finished (0),
	m_next_not_finished (0) {}
    
    void addTask (Task* task);
    void addNext (Pipeline* next);
    void start ();

    void wait () {
	if (m_started)
	    m_thread.join ();
    }
};

} /* namespace yann */

#endif /* YANN_PIPELINE_H */
